"""Tasks for canvas"""
import hashlib
import logging
from functools import partial

from celery import shared_task
from lms.djangoapps.instructor_task.api_helper import submit_task
from lms.djangoapps.instructor_task.tasks_base import BaseInstructorTask
from lms.djangoapps.instructor_task.tasks_helper.runner import run_main_task
from ol_openedx_canvas_integration import task_helpers
from ol_openedx_canvas_integration.constants import (
    TASK_TYPE_PUSH_EDX_GRADES_TO_CANVAS,
    TASK_TYPE_SYNC_CANVAS_ENROLLMENTS,
)

TASK_LOG = logging.getLogger("edx.celery.task")


def run_sync_canvas_enrollments(
    request, course_key, canvas_course_id, unenroll_current
):
    """
    Submit a task to start syncing canvas enrollments
    """
    task_type = TASK_TYPE_SYNC_CANVAS_ENROLLMENTS
    task_class = sync_canvas_enrollments_task
    task_input = {
        "course_key": course_key,
        "canvas_course_id": canvas_course_id,
        "unenroll_current": unenroll_current,
    }
    task_key = hashlib.md5(course_key.encode("utf8")).hexdigest()  # noqa: S324
    TASK_LOG.debug("Submitting task to sync canvas enrollments")
    return submit_task(request, task_type, task_class, course_key, task_input, task_key)


@shared_task(base=BaseInstructorTask)
def sync_canvas_enrollments_task(entry_id, xmodule_instance_args):
    """
    Fetch enrollments from canvas and update
    """
    action_name = "sync_canvas_enrollments"
    TASK_LOG.info("Running task to sync Canvas enrollments")
    task_fn = partial(task_helpers.sync_canvas_enrollments, xmodule_instance_args)
    return run_main_task(entry_id, task_fn, action_name)


def run_push_edx_grades_to_canvas(request, course_id):
    """
    Submit a task to start pushing edX grades to Canvas
    """
    task_type = TASK_TYPE_PUSH_EDX_GRADES_TO_CANVAS
    task_class = push_edx_grades_to_canvas_task
    task_input = {
        # course_key is already passed into the task, but we need to put it in task_input as well  # noqa: E501
        # so the instructor task status can be properly calculated instead of being marked incomplete  # noqa: E501
        "course_key": str(course_id)
    }
    task_key = hashlib.md5(course_id.encode("utf8")).hexdigest()  # noqa: S324
    TASK_LOG.debug("Submitting task to push edX grades to Canvas")
    return submit_task(request, task_type, task_class, course_id, task_input, task_key)


@shared_task(base=BaseInstructorTask)
def push_edx_grades_to_canvas_task(entry_id, xmodule_instance_args):
    """
    Push edX grades to Canvas
    """
    action_name = "push_edx_grades_to_canvas"
    TASK_LOG.info("Running task to push edX grades to Canvas")
    task_fn = partial(task_helpers.push_edx_grades_to_canvas, xmodule_instance_args)
    return run_main_task(entry_id, task_fn, action_name)



"""
Helper functions for canvas integration tasks
"""
from time import time

from lms.djangoapps.courseware.courses import get_course_by_id
from lms.djangoapps.instructor_task.tasks_helper.runner import TaskProgress
from ol_openedx_canvas_integration import api


def sync_canvas_enrollments(
    _xmodule_instance_args,
    _entry_id,
    course_id,  # noqa: ARG001
    task_input,
    action_name,
):
    """Partial function to sync canvas enrollments"""
    start_time = time()
    num_reports = 1
    task_progress = TaskProgress(action_name, num_reports, start_time)
    api.sync_canvas_enrollments(
        course_key=task_input["course_key"],
        canvas_course_id=task_input["canvas_course_id"],
        unenroll_current=task_input["unenroll_current"],
    )
    # for simplicity, only one task update for now when everything is done
    return task_progress.update_task_state(extra_meta={"step": "Done"})


def push_edx_grades_to_canvas(
    _xmodule_instance_args,
    _entry_id,
    course_id,
    task_input,  # noqa: ARG001
    action_name,
):
    """Partial function to push edX grades to canvas"""
    start_time = time()
    num_reports = 1
    task_progress = TaskProgress(action_name, num_reports, start_time)
    course = get_course_by_id(course_id)
    grades_updated, assignments_created = api.push_edx_grades_to_canvas(course=course)
    results = {"grades": len(grades_updated), "assignments": len(assignments_created)}
    return task_progress.update_task_state(
        extra_meta={"step": "Done", "results": results}
    )



"""
Canvas Integration Application Configuration
"""

from django.apps import AppConfig
from edx_django_utils.plugins import PluginContexts, PluginSettings, PluginURLs
from lms.djangoapps.instructor.constants import INSTRUCTOR_DASHBOARD_PLUGIN_VIEW_NAME
from openedx.core.constants import COURSE_ID_PATTERN
from openedx.core.djangoapps.plugins.constants import ProjectType, SettingsType


class CanvasIntegrationConfig(AppConfig):
    """
    Configuration class for Canvas integration app
    """

    name = "ol_openedx_canvas_integration"

    plugin_app = {
        PluginURLs.CONFIG: {
            ProjectType.LMS: {
                PluginURLs.NAMESPACE: "",
                PluginURLs.REGEX: f"courses/{COURSE_ID_PATTERN}/canvas/api/",
                PluginURLs.RELATIVE_PATH: "urls",
            }
        },
        PluginSettings.CONFIG: {
            ProjectType.LMS: {
                SettingsType.PRODUCTION: {
                    PluginSettings.RELATIVE_PATH: "settings.production"
                },
                SettingsType.COMMON: {PluginSettings.RELATIVE_PATH: "settings.common"},
            }
        },
        PluginContexts.CONFIG: {
            ProjectType.LMS: {
                INSTRUCTOR_DASHBOARD_PLUGIN_VIEW_NAME: "ol_openedx_canvas_integration.context_api.plugin_context"  # noqa: E501
            }
        },
    }




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/ol_openedx_canvas_integration:canvas_integration_package

from setuptools import setup

setup(**{
    'description': 'An Open edX plugin to add canvas integration support',
    'entry_points': {
        'lms.djangoapp': [
            'ol_openedx_canvas_integration = ol_openedx_canvas_integration.app:CanvasIntegrationConfig',
        ],
    },
    'install_requires': (
        'Django>2.0',
        'celery>=4.4.7',
        'edx-django-utils>4.0.0',
    ),
    'license': 'BSD-3-Clause',
    'name': 'ol-openedx-canvas-integration',
    'namespace_packages': (
    ),
    'package_data': {
        'ol_openedx_canvas_integration': (
            'static/js/canvas_integration.js',
            'templates/canvas_integration.html',
        ),
    },
    'packages': (
        '',
        'ol_openedx_canvas_integration',
        'ol_openedx_canvas_integration.settings',
    ),
    'python_requires': '>=3.8',
    'version': '0.3.0',
})
